import { Chatty } from '@looker/chatty';

// Given an object `Target`, find all property names of type `Type`
// Given an object `Target`, filter out all properties that aren't of type `Type`
function createElement(name) {
  let props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  let children = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  const element = document.createElement(name);

  for (const [name, value] of Object.entries(props)) {
    if (name === 'style') {
      Object.assign(element.style, props.style);
    } else {
      element.setAttribute(name, value);
    }
  }

  for (const child of Array.isArray(children) ? children : [children]) {
    element.append(child);
  }

  return element;
}

/**
 * Constructs the chart iframe URL from the baseUrl, chartId & tenantId
 */
const getChartUrl = options => {
  try {
    const url = new URL(options.baseUrl);
    url.pathname = [url.pathname, url.pathname.slice(-1) === '/' ? '' : '/', 'embed/charts'].join('');
    url.search = "id=".concat(options.chartId, "&sdk=1");
    url.search += options.autorefresh ? "&autorefresh=".concat(options.autorefresh) : '';
    url.search += options.filter ? "&filter=".concat(encodeURIComponent(JSON.stringify(options.filter))) : '';
    return url.toString();
  } catch (e) {
    throw new Error('Base URL must be a valid URL');
  }
};
/* Parses a CSS Measurement from an unknown value
     - if it's a string, we trust that it is well-formed
     - if it's a number, we assume the units are pixels
     - otherwise we return null
*/

const parseCSSMeasurement = value => {
  if (typeof value === 'string') return value;
  if (typeof value === 'number') return "".concat(value, "px");
  return null;
};

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
class EmbeddedChart {
  constructor(element, iframe, connection, options) {
    _defineProperty(this, "options", void 0);

    _defineProperty(this, "connection", void 0);

    _defineProperty(this, "iframe", void 0);

    _defineProperty(this, "element", void 0);

    this.element = element;
    this.iframe = iframe;
    this.connection = connection;
    this.options = _objectSpread({}, options);
  }

  async refresh() {
    return await this.connection.sendAndReceive('refresh');
  }

  async set(name, value) {
    if (name === 'autorefresh') {
      if (typeof value === 'number') {
        return await this.connection.sendAndReceive('set', name, value);
      }

      return Promise.reject('autorefresh property value should be a number');
    }

    return Promise.reject('Unknown property name');
  }

  async get(name) {
    if (name === 'autorefresh') {
      const [result] = await this.connection.sendAndReceive('get', name);
      return result;
    }

    return Promise.reject('Unknown property name');
  }

}

const getChartOptions = options => {
  if (typeof options !== 'object' || options === null) {
    throw new Error('Options argument must be an object');
  }

  const {
    baseUrl,
    chartId,
    filter,
    autorefresh,
    width,
    height
  } = options;

  if (typeof baseUrl !== 'string' || baseUrl.length === 0) {
    throw new Error('Base URL must be a valid URL');
  }

  if (typeof chartId !== 'string' || chartId.length === 0) {
    throw new Error('Chart ID must be specified');
  }

  if (width !== undefined && !['number', 'string'].includes(typeof width)) {
    throw new Error('Width must be a string or number if specified');
  }

  if (height !== undefined && !['number', 'string'].includes(typeof height)) {
    throw new Error('Height must be a string or number if specified');
  }

  if (filter !== undefined && (!filter || typeof filter !== 'object')) {
    throw new Error('Filter must be an object if specified');
  }

  if (autorefresh !== undefined && typeof autorefresh !== 'number') {
    throw new Error('Autorefresh interval must be an object if specified');
  }

  return {
    baseUrl,
    chartId,
    filter,
    autorefresh,
    width,
    height
  };
};

const embedChart = async (unsafeOptions, container) => {
  if (!(container instanceof HTMLElement)) {
    throw new Error('Target container must be a HTML element');
  }

  const options = getChartOptions(unsafeOptions); // Remove any existing nodes in our target container

  while (container.firstChild) container.removeChild(container.firstChild);

  const chartUrl = getChartUrl(options); // Create styled container

  const embedRoot = createElement('div', {
    style: {
      position: 'relative',
      overflow: 'hidden',
      width: parseCSSMeasurement(options.width) || '100%',
      height: parseCSSMeasurement(options.height) || '100%'
    }
  }); // Create iframe

  const host = Chatty.createHost(chartUrl).withSandboxAttribute('allow-scripts').withSandboxAttribute('allow-same-origin').appendTo(embedRoot).build(); // Customise IFrame styles

  Object.assign(host.iframe.style, {
    position: 'absolute',
    top: 0,
    left: 0,
    width: '100%',
    height: '100%',
    border: 0
  });
  container.appendChild(embedRoot);
  const connection = await host.connect();
  return new EmbeddedChart(embedRoot, host.iframe, connection, options);
};
var index = {
  embedChart
};

export default index;
export { EmbeddedChart, embedChart };
